// mngplg.c  -- plugin-specific stuff

#include <windows.h>
#include "mngctrl.h"
#include "npapidefs.h"

static NPNetscapeFuncs* g_pNavigatorFuncs;
static NPPluginFuncs* g_pluginFuncs;
static HMODULE g_hInst;

/* ----------------------------------------------------------------------- */


const char* PLGSERVICE_UserAgent(PLGINST plginst)
{
    return g_pNavigatorFuncs->uagent(plginst->npp);
}

PLGERRCODE PLGSERVICE_GetURL(PLGINST plginst, const char *url, const char *target)
{
    return g_pNavigatorFuncs->geturl(plginst->npp, url, target);
}

/* ----------------------------------------------------------------------- */


BOOL APIENTRY DllMain(HANDLE hModule, DWORD reason, LPVOID lpReserved)
{
    switch(reason) {
    case DLL_PROCESS_ATTACH:
		g_hInst=hModule;
		break;
    case DLL_THREAD_ATTACH:
		break;
    case DLL_THREAD_DETACH:
		break;
    case DLL_PROCESS_DETACH:
		break;
    }
    return TRUE;
}

/////////////////////
/////////////////////  low-level plug-in NPAPI functions

static NPError NPP_New(NPMIMEType pluginType,NPP instance,uint16 mode,
		int16 argc,char* argn[],char* argv[],NPSavedData* saved)
{
	int i;
	PLGINST plginst;
	NPError errcode;
	if(!instance) return NPERR_INVALID_INSTANCE_ERROR;

	plginst = calloc(sizeof(PLGINST_t),1);
	if(!plginst) return NPERR_OUT_OF_MEMORY_ERROR;

	instance->pdata = (void*)plginst;
	plginst->npp = instance;

	errcode= (NPError) PLG_New(plginst,(int)mode,0,(COLORREF)0);
	if(errcode!=NPERR_NO_ERROR) return errcode;

	for(i=0;i<argc;i++) {
		PLG_Param(plginst,argn[i],argv[i]);
	}
	PLG_ParamDone(plginst);

	return NPERR_NO_ERROR;
}

static NPError NPP_Destroy(NPP instance, NPSavedData** save)
{
	PLGINST plginst;
	NPError ret;
	if(!instance) return NPERR_INVALID_INSTANCE_ERROR;
	plginst = (PLGINST)instance->pdata;
	if(!plginst) return NPERR_INVALID_INSTANCE_ERROR;
	ret=(NPError)PLG_Destroy(plginst);
	free(plginst);
	instance->pdata=NULL;
	return ret;
}


static NPError NPP_SetWindow(NPP instance, NPWindow* window)
{
	HWND newhwnd,oldhwnd;
	PLGINST plginst;
	plginst = (PLGINST)instance->pdata;
	if(!plginst) return NPERR_INVALID_INSTANCE_ERROR;

	newhwnd = (HWND)window->window;
	oldhwnd = plginst->hwnd;

	if(oldhwnd==newhwnd) {
		// We could redraw the window, but it should be okay just
		// to ignore this.
		return NPERR_NO_ERROR;
	}

	if(oldhwnd) {
		// get rid of the old window
		PLG_DestroyWindow(plginst);
	}

	if(newhwnd) {
		PLG_NewWindow(plginst,newhwnd);
	}

	plginst->hwnd = newhwnd;
	return NPERR_NO_ERROR;
}


static NPError NPP_NewStream(NPP instance,NPMIMEType type,NPStream *stream, 
	      NPBool seekable,uint16 *stype)
{
	NPError ret;
	PLGINST plginst;
	unsigned int size;
	plginst = (PLGINST)instance->pdata;
	if(!plginst) return NPERR_INVALID_INSTANCE_ERROR;

	size = stream->end;
	if(size>0x7fffffff) size=0;

	ret=(NPError)PLG_NewStream(plginst,(void*)stream,stream->url,size);

	(*stype)=NP_NORMAL;
	return ret;
}

static int32 NPP_WriteReady(NPP instance, NPStream *stream)
{
	/* Number of bytes ready to accept in NPP_Write() */
	/* We can handle any amount, so just return some really big number. */
	return (int32)0X0FFFFFFF;
}

static int32 NPP_Write(NPP instance, NPStream *stream, int32 offset, int32 len, void *buffer)
{
	PLGERRCODE errcode;
	PLGINST plginst;

	plginst = (PLGINST)instance->pdata;
	if(!plginst) return NPERR_INVALID_INSTANCE_ERROR;

	errcode=PLG_Write(plginst,(void*)stream,(int)offset,(int)len,buffer);
	if(errcode!=PLGERR_NO_ERROR) return -1;
	return len;
}

static NPError NPP_DestroyStream(NPP instance, NPStream *stream, NPReason reason)
{
	PLGINST plginst;
	plginst = (PLGINST)instance->pdata;
	if(!plginst) return NPERR_INVALID_INSTANCE_ERROR;
	return (NPError)PLG_DestroyStream(plginst,(void*)stream,(int)reason);
}

static void NPP_StreamAsFile(NPP instance, NPStream *stream, const char* fname)
{
	return;
}

static void NPP_Print(NPP instance, NPPrint* printInfo)
{
	PLGINST plginst;
	HDC pdc;

	plginst = (PLGINST)instance->pdata;
	if(!plginst) return;

	if(!printInfo) {
		// browser probably wants us to print full-page
		PLG_FullPrint(plginst);
		return;
	}

	if(printInfo->mode==NP_FULL) {
		PLG_FullPrint(plginst);
		return;
	}
	else if(printInfo->mode==NP_EMBED) {
		NPWindow* w;

		// fixme: make this work for API version<0.11
		pdc= (HDC) (printInfo->print.embedPrint.platformPrint);

		w= &(printInfo->print.embedPrint.window);

		PLG_EmbedPrint(plginst,pdc,
			w->x,w->y,w->width,w->height);
	}
}

/*+++++++++++++++++++++++++++++++++++++++++++++++++
 * NPP_URLNotify:
 * Notifies the instance of the completion of a URL request. 
 +++++++++++++++++++++++++++++++++++++++++++++++++*/
static void NPP_URLNotify(NPP instance, const char* url, NPReason reason, void* notifyData)
{
	return;
}


static JRIGlobalRef NPP_GetJavaClass(void)
{
    return NULL;
}

NPError WINAPI NP_GetEntryPoints(NPPluginFuncs* pFuncs)
{
    if(!pFuncs) return NPERR_INVALID_FUNCTABLE_ERROR;

    pFuncs->version       = (NP_VERSION_MAJOR << 8) | NP_VERSION_MINOR;
    pFuncs->newp          = NPP_New;
    pFuncs->destroy       = NPP_Destroy;
    pFuncs->setwindow     = NPP_SetWindow;
    pFuncs->newstream     = NPP_NewStream;
    pFuncs->destroystream = NPP_DestroyStream;
    pFuncs->asfile        = NPP_StreamAsFile;
    pFuncs->writeready    = NPP_WriteReady;
    pFuncs->write         = NPP_Write;
    pFuncs->print         = NPP_Print;
    pFuncs->event         = NULL;

	g_pluginFuncs		  = pFuncs;

    return NPERR_NO_ERROR;
}

NPError WINAPI NP_Initialize(NPNetscapeFuncs* pFuncs)
{
	int navMinorVers;

    if(!pFuncs) return NPERR_INVALID_FUNCTABLE_ERROR;

    g_pNavigatorFuncs = pFuncs; // save it for future reference 

    if(HIBYTE(pFuncs->version) > NP_VERSION_MAJOR)
        return NPERR_INCOMPATIBLE_VERSION_ERROR;

    navMinorVers = g_pNavigatorFuncs->version & 0xFF;
	if(navMinorVers>=NPVERS_HAS_NOTIFICATION)
		g_pluginFuncs->urlnotify = NPP_URLNotify;
	if( navMinorVers>=NPVERS_HAS_LIVECONNECT)
		g_pluginFuncs->javaClass = NPP_GetJavaClass();

    return PLG_Initialize(g_hInst,g_hInst,PLGARCH_NETSCAPE,
		HIBYTE(g_pNavigatorFuncs->version),
		g_pNavigatorFuncs->version & 0xFF);
}

NPError WINAPI NP_Shutdown(void)
{
    PLG_Shutdown();

    g_pNavigatorFuncs = NULL;
    return NPERR_NO_ERROR;
}
