// mngsv.c
//
// Standalone MNG image viewer, using code from MNGPLG plug-in
// By Jason Summers.

#include <windows.h>
#include <string.h>
#include <stdio.h>

#include "mngctrl.h"

#include "resourcesv.h"

#define WM_MNGSV_LOADFILE  (WM_USER+1)

static char g_filename[MAX_PATH+1];
static HINSTANCE g_hInst;
static HWND g_hwnd;
static PLGINST_t plginst;

static LRESULT CALLBACK WndProcMain(HWND hwnd, UINT msg, WPARAM wParam, LPARAM lParam);



PLGERRCODE PLGSERVICE_GetURL(PLGINST plginst, const char *url, const char *target)
{
	return PLGERR_GENERIC_ERROR;
}


int WINAPI WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance,
   LPSTR lpCmdLine,int nCmdShow)
{
	MSG msg;
	WNDCLASS wc;
	HACCEL hAccTable;
	int i;

	g_hInst=hInstance;
	strcpy(g_filename,"");

	// command-line argument processing.
	// We handle one file only, with or without quotes.
	if(lpCmdLine[0]) {
		if(lpCmdLine[0]=='"') {
			lstrcpyn(g_filename,&lpCmdLine[1],MAX_PATH);
			i=strlen(g_filename);
			if(g_filename[i-1]=='"') g_filename[i-1]='\0';
		}
		else {
			lstrcpyn(g_filename,lpCmdLine,MAX_PATH);
		}
	}

	ZeroMemory(&wc,sizeof(WNDCLASS));
	wc.style = 0;
	wc.lpfnWndProc = (WNDPROC)WndProcMain;
	wc.cbClsExtra = 0;
	wc.cbWndExtra = 0;
	wc.hInstance = g_hInst;
	wc.hIcon = LoadIcon(g_hInst,"MNGCTRLICON1");
	wc.hCursor = LoadCursor(NULL, IDC_ARROW);
	wc.hbrBackground = GetStockObject(WHITE_BRUSH);
	wc.lpszMenuName =  "MNGSVMENU";
	wc.lpszClassName = "MNGSVCLASS";
	if(!RegisterClass(&wc)) {
		return 0;
	}

	g_hwnd = CreateWindow(
		"MNGSVCLASS","MNGSV",
		WS_VISIBLE|WS_OVERLAPPEDWINDOW|WS_CLIPCHILDREN,
		CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,CW_USEDEFAULT,
		NULL,NULL,g_hInst,NULL);
	if (!g_hwnd) return 0;


	plginst.ctl=NULL;  // nonzero if PLG_New called
	plginst.hwnd=NULL; // nonzero if PLG_NewWindow called
	PLG_Initialize(g_hInst,g_hInst,PLGARCH_STANDALONE,1,0);

	DragAcceptFiles(g_hwnd,TRUE);
	hAccTable=LoadAccelerators(g_hInst,"ACCELTABLE");

	if(strlen(g_filename)) {
		PostMessage(g_hwnd,WM_MNGSV_LOADFILE,0,0);
	}

	while(GetMessage(&msg,NULL,0,0)) {
		if(!TranslateAccelerator(g_hwnd,hAccTable,&msg)) {
			TranslateMessage(&msg);
			DispatchMessage(&msg);
		}
	}

	PLG_Shutdown();

	return 0;
}


static void mngsv_destroyinstance(void)
{
	if(!plginst.ctl) return;

	if(plginst.hwnd) {
		PLG_DestroyWindow(&plginst);
		plginst.hwnd=NULL;
	}

	PLG_Destroy(&plginst);
	plginst.ctl=NULL;
}

static void mngsv_loadfile(HWND hwnd)
{
	HANDLE fh;
	unsigned char fbuf[32768];
	int i,n;
	int offset;
	DWORD fsize,bytesread;
	char buf[MAX_PATH+100];
	int baseindex;

	fh=INVALID_HANDLE_VALUE;

	mngsv_destroyinstance(); // unload previous

	if(!strlen(g_filename)) return;

	fh=CreateFile(g_filename,GENERIC_READ,FILE_SHARE_READ,NULL,
		OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if(fh==INVALID_HANDLE_VALUE) {
		return;
	}

	fsize=GetFileSize(fh,NULL);

	plginst.pdata=NULL;
	plginst.ctl=(void*)1;
	PLG_New(&plginst,PLGMODE_FULL,0,(COLORREF)0);
	PLG_ParamDone(&plginst);

	plginst.hwnd=g_hwnd;
	PLG_NewWindow(&plginst,plginst.hwnd);

	PLG_NewStream(&plginst, (void*)1, g_filename, fsize);

	offset=0;

	ReadFile(fh,fbuf,32768,&bytesread,NULL);
	n=(int)bytesread;
	while(n>0) {
		PLG_Write(&plginst,(void*)1,offset,n,fbuf);
		offset+=n;
		ReadFile(fh,fbuf,32768,&bytesread,NULL);
		n=(int)bytesread;
	}
	PLG_DestroyStream(&plginst,(void*)1,PLGRES_DONE);

	if(fh!=INVALID_HANDLE_VALUE) CloseHandle(fh);

	// set window title
	// find base file name
	baseindex=0;
	for(i=0;g_filename[i];i++) {
		if(g_filename[i]=='\\') baseindex=i+1;
	}

	wsprintf(buf,"%s - MNGSV",&g_filename[baseindex]);
	SetWindowText(g_hwnd,buf);
}

static void dropped_files(HWND hwnd, HDROP hDrop)
{
	UINT ret;
	DWORD attr;
	char fn[MAX_PATH];

	ret=DragQueryFile(hDrop,0xFFFFFFFF,NULL,0);
	if(ret==1) {
		ret=DragQueryFile(hDrop,0,fn,MAX_PATH);
		if(ret) {
			attr=GetFileAttributes(fn);
			if((attr==0xffffffff) || !(attr&FILE_ATTRIBUTE_DIRECTORY)) {
				strcpy(g_filename,fn);
				mngsv_loadfile(hwnd);
			}
		}
	}
	DragFinish(hDrop);
}

static void mngsv_openfiledlg(HWND hwnd)
{
	OPENFILENAME ofn;
	char fn[MAX_PATH];

	strcpy(fn,"");
	ZeroMemory(&ofn,sizeof(OPENFILENAME));

	ofn.lStructSize=sizeof(OPENFILENAME);
	ofn.hwndOwner=hwnd;
	ofn.lpstrFilter="MNG, JNG, PNG\0*.mng;*.jng;*.png\0All files\0*.*\0";
	ofn.nFilterIndex=1;

	ofn.lpstrTitle=NULL;
	ofn.lpstrInitialDir=NULL;
	ofn.lpstrFile=fn;
	ofn.nMaxFile=MAX_PATH;
	ofn.Flags=OFN_FILEMUSTEXIST|OFN_HIDEREADONLY;

	if(!GetOpenFileName(&ofn)) {
		return;
	}

	strcpy(g_filename,fn);
	mngsv_loadfile(hwnd);
}

static LRESULT CALLBACK WndProcMain(HWND hwnd, UINT msg,
     WPARAM wParam, LPARAM lParam)
{
	WORD id;

	id=LOWORD(wParam);

	switch(msg) {

	case WM_MNGSV_LOADFILE:
		mngsv_loadfile(hwnd);
		break;

	case WM_DESTROY:
		mngsv_destroyinstance();
		PostQuitMessage(0);

	case WM_INITMENU:
		{
			HMENU m;
			int anim;
			m=(HMENU)wParam;
			anim=script_GetAnimateState(&plginst);
			CheckMenuItem(m,IDM_ANIMATE,(anim?MF_CHECKED:MF_UNCHECKED));
		}
		return 0;

	case WM_DROPFILES:
		dropped_files(hwnd,(HDROP)wParam);
		return 0;

	case WM_COMMAND:
		switch(id) {
		case IDM_OPEN:
			mngsv_openfiledlg(hwnd);
			return 0;
		case IDM_ANIMATE:
			if(script_GetAnimateState(&plginst))
				script_Animate(&plginst,0);
			else
				script_Animate(&plginst,1);
			return 0;
		case IDM_COPY:
			script_CopyImage(&plginst);
			return 0;
		case IDM_PROPERTIES:
			script_ImageProperties(&plginst);
			return 0;
		case IDM_ABOUT:
			script_AboutBox(&plginst);
			return 0;
		case IDM_EXIT:
			DestroyWindow(hwnd);
			return 0;
		}
	}

	return DefWindowProc(hwnd, msg, wParam, lParam);
}
